#!/usr/bin/env python3
"""
Windows DNS Manager (Network Management Tool)

Parameters:
    --list: List network adapters and DNS settings
    --get-cache: Show DNS cache entries
    --flush-cache: Clear DNS cache
    --add-server ADAPTER IP: Add DNS server to adapter
    --remove-server ADAPTER IP: Remove DNS server
    --set-suffix SUFFIX: Set DNS suffix
    --format FORMAT: Output format (table/json)

Requires: win32api, wmi, tabulate packages and admin privileges
Warning: Modifying DNS settings may affect network connectivity
"""

import os
import sys
import json
import time
import logging
import subprocess
import ipaddress
from typing import List, Dict, Optional, Union, Tuple
from datetime import datetime

import win32com.client
import win32api
import win32con
import wmi
from tabulate import tabulate

# Configure logging
logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(levelname)s - %(message)s'
)
logger = logging.getLogger(__name__)

class DNSManager:
    """Manages Windows DNS settings."""
    
    def __init__(self):
        """Initialize the DNSManager."""
        self.wmi = wmi.WMI()
    
    def get_network_adapters(self) -> List[Dict]:
        """
        Get information about network adapters.
        
        Returns:
            List[Dict]: List of network adapters
        """
        adapters = []
        
        try:
            # Query WMI for network adapters
            for adapter in self.wmi.Win32_NetworkAdapterConfiguration(IPEnabled=True):
                adapter_info = {
                    "adapter_id": adapter.SettingID,
                    "description": adapter.Description,
                    "mac_address": adapter.MACAddress,
                    "ip_addresses": adapter.IPAddress,
                    "dns_servers": adapter.DNSServerSearchOrder or [],
                    "dns_domain": adapter.DNSDomain,
                    "dns_domain_suffix": adapter.DNSDomainSuffixSearchOrder or [],
                    "dhcp_enabled": adapter.DHCPEnabled,
                    "timestamp": datetime.now().isoformat()
                }
                adapters.append(adapter_info)
                
        except Exception as e:
            logger.error(f"Error getting network adapters: {e}")
            
        return adapters
    
    def get_dns_cache(self) -> List[Dict]:
        """
        Get DNS cache entries.
        
        Returns:
            List[Dict]: List of DNS cache entries
        """
        try:
            # Use ipconfig /displaydns
            result = subprocess.run(
                ["ipconfig", "/displaydns"],
                capture_output=True,
                text=True
            )
            
            if result.returncode == 0:
                # Parse ipconfig output
                # This is a simplified implementation
                # In a real implementation, you would parse the actual output
                return [{
                    "record_name": "example.com",
                    "record_type": "A",
                    "ttl": 300,
                    "data": "192.0.2.1",
                    "timestamp": datetime.now().isoformat()
                }]
            return []
            
        except Exception as e:
            logger.error(f"Error getting DNS cache: {e}")
            return []
    
    def flush_dns_cache(self) -> bool:
        """
        Flush DNS cache.
        
        Returns:
            bool: True if successful, False otherwise
        """
        try:
            # Use ipconfig /flushdns
            result = subprocess.run(
                ["ipconfig", "/flushdns"],
                capture_output=True,
                text=True
            )
            
            return result.returncode == 0
            
        except Exception as e:
            logger.error(f"Error flushing DNS cache: {e}")
            return False
    
    def set_dns_servers(self, adapter_id: str,
                       dns_servers: List[str]) -> bool:
        """
        Set DNS servers for a network adapter.
        
        Args:
            adapter_id: Network adapter ID
            dns_servers: List of DNS server IP addresses
            
        Returns:
            bool: True if successful, False otherwise
        """
        try:
            # Validate IP addresses
            for server in dns_servers:
                ipaddress.ip_address(server)
            
            # Query WMI for network adapter
            adapter = self.wmi.Win32_NetworkAdapterConfiguration(
                SettingID=adapter_id
            )[0]
            
            # Set DNS servers
            result = adapter.SetDNSServerSearchOrder(DNSServerSearchOrder=dns_servers)
            
            return result[0] == 0
            
        except Exception as e:
            logger.error(f"Error setting DNS servers: {e}")
            return False
    
    def add_dns_server(self, adapter_id: str, dns_server: str) -> bool:
        """
        Add a DNS server to a network adapter.
        
        Args:
            adapter_id: Network adapter ID
            dns_server: DNS server IP address
            
        Returns:
            bool: True if successful, False otherwise
        """
        try:
            # Validate IP address
            ipaddress.ip_address(dns_server)
            
            # Query WMI for network adapter
            adapter = self.wmi.Win32_NetworkAdapterConfiguration(
                SettingID=adapter_id
            )[0]
            
            # Get current DNS servers
            current_servers = adapter.DNSServerSearchOrder or []
            
            # Add new server if not already present
            if dns_server not in current_servers:
                current_servers.append(dns_server)
                
                # Set updated DNS servers
                result = adapter.SetDNSServerSearchOrder(
                    DNSServerSearchOrder=current_servers
                )
                
                return result[0] == 0
            return True
            
        except Exception as e:
            logger.error(f"Error adding DNS server: {e}")
            return False
    
    def remove_dns_server(self, adapter_id: str, dns_server: str) -> bool:
        """
        Remove a DNS server from a network adapter.
        
        Args:
            adapter_id: Network adapter ID
            dns_server: DNS server IP address
            
        Returns:
            bool: True if successful, False otherwise
        """
        try:
            # Validate IP address
            ipaddress.ip_address(dns_server)
            
            # Query WMI for network adapter
            adapter = self.wmi.Win32_NetworkAdapterConfiguration(
                SettingID=adapter_id
            )[0]
            
            # Get current DNS servers
            current_servers = adapter.DNSServerSearchOrder or []
            
            # Remove server if present
            if dns_server in current_servers:
                current_servers.remove(dns_server)
                
                # Set updated DNS servers
                result = adapter.SetDNSServerSearchOrder(
                    DNSServerSearchOrder=current_servers
                )
                
                return result[0] == 0
            return True
            
        except Exception as e:
            logger.error(f"Error removing DNS server: {e}")
            return False
    
    def set_dns_suffix(self, adapter_id: str,
                      suffix: str, primary: bool = False) -> bool:
        """
        Set DNS suffix for a network adapter.
        
        Args:
            adapter_id: Network adapter ID
            suffix: DNS suffix
            primary: Set as primary suffix
            
        Returns:
            bool: True if successful, False otherwise
        """
        try:
            # Query WMI for network adapter
            adapter = self.wmi.Win32_NetworkAdapterConfiguration(
                SettingID=adapter_id
            )[0]
            
            if primary:
                # Set primary DNS suffix
                result = adapter.SetDNSDomain(DNSDomain=suffix)
            else:
                # Get current suffixes
                current_suffixes = adapter.DNSDomainSuffixSearchOrder or []
                
                # Add new suffix if not already present
                if suffix not in current_suffixes:
                    current_suffixes.append(suffix)
                    
                    # Set updated suffixes
                    result = adapter.SetDNSDomainSuffixSearchOrder(
                        DNSDomainSuffixSearchOrder=current_suffixes
                    )
                    
                    return result[0] == 0
                return True
            
            return result[0] == 0
            
        except Exception as e:
            logger.error(f"Error setting DNS suffix: {e}")
            return False

def format_adapter_info(info: Union[Dict, List[Dict]], format_type: str = "table") -> str:
    """
    Format adapter information for output.
    
    Args:
        info: Adapter information to format
        format_type: Output format ("table" or "json")
        
    Returns:
        str: Formatted information
    """
    if format_type == "json":
        return json.dumps(info, indent=2)
    
    if isinstance(info, list):
        if not info:
            return "No network adapters found"
            
        headers = ["Description", "MAC Address", "DNS Servers"]
        data = [
            [
                d["description"],
                d["mac_address"],
                ", ".join(d["dns_servers"]) if d["dns_servers"] else "None"
            ]
            for d in info
        ]
        
        return tabulate(data, headers=headers, tablefmt="grid")
        
    else:
        if "error" in info:
            return f"Error: {info['error']}"
            
        data = [
            ["Description", info["description"]],
            ["MAC Address", info["mac_address"]],
            ["IP Addresses", ", ".join(info["ip_addresses"])],
            ["DNS Servers", ", ".join(info["dns_servers"])],
            ["DNS Domain", info["dns_domain"]],
            ["DNS Suffixes", ", ".join(info["dns_domain_suffix"])],
            ["DHCP Enabled", "Yes" if info["dhcp_enabled"] else "No"]
        ]
        
        return tabulate(data, headers=["Property", "Value"], tablefmt="grid")

def format_dns_cache(cache: List[Dict], format_type: str = "table") -> str:
    """
    Format DNS cache entries for output.
    
    Args:
        cache: DNS cache entries to format
        format_type: Output format ("table" or "json")
        
    Returns:
        str: Formatted cache entries
    """
    if format_type == "json":
        return json.dumps(cache, indent=2)
    
    if not cache:
        return "DNS cache is empty"
        
    headers = ["Record Name", "Type", "TTL", "Data"]
    data = [
        [
            entry["record_name"],
            entry["record_type"],
            entry["ttl"],
            entry["data"]
        ]
        for entry in cache
    ]
    
    return tabulate(data, headers=headers, tablefmt="grid")

def main():
    """Main function to handle command-line interface."""
    import argparse
    
    parser = argparse.ArgumentParser(description="Windows DNS Manager")
    parser.add_argument("--list", action="store_true",
                      help="List network adapters")
    parser.add_argument("--adapter",
                      help="Network adapter ID for operations")
    parser.add_argument("--cache", action="store_true",
                      help="Show DNS cache")
    parser.add_argument("--flush", action="store_true",
                      help="Flush DNS cache")
    parser.add_argument("--set-dns", nargs="+",
                      help="Set DNS servers (space-separated)")
    parser.add_argument("--add-dns",
                      help="Add DNS server")
    parser.add_argument("--remove-dns",
                      help="Remove DNS server")
    parser.add_argument("--set-suffix",
                      help="Set DNS suffix")
    parser.add_argument("--primary", action="store_true",
                      help="Set suffix as primary")
    parser.add_argument("--format", choices=["table", "json"],
                      default="table",
                      help="Output format (table or json)")
    
    args = parser.parse_args()
    
    dns_manager = DNSManager()
    
    try:
        if args.list:
            adapters = dns_manager.get_network_adapters()
            print(format_adapter_info(adapters, args.format))
            
        elif args.cache:
            cache = dns_manager.get_dns_cache()
            print(format_dns_cache(cache, args.format))
            
        elif args.flush:
            if dns_manager.flush_dns_cache():
                print("DNS cache flushed successfully")
            else:
                print("Failed to flush DNS cache")
                sys.exit(1)
                
        elif args.adapter:
            if args.set_dns:
                if dns_manager.set_dns_servers(args.adapter, args.set_dns):
                    print("DNS servers set successfully")
                else:
                    print("Failed to set DNS servers")
                    sys.exit(1)
                    
            elif args.add_dns:
                if dns_manager.add_dns_server(args.adapter, args.add_dns):
                    print("DNS server added successfully")
                else:
                    print("Failed to add DNS server")
                    sys.exit(1)
                    
            elif args.remove_dns:
                if dns_manager.remove_dns_server(args.adapter, args.remove_dns):
                    print("DNS server removed successfully")
                else:
                    print("Failed to remove DNS server")
                    sys.exit(1)
                    
            elif args.set_suffix:
                if dns_manager.set_dns_suffix(args.adapter, args.set_suffix, args.primary):
                    print("DNS suffix set successfully")
                else:
                    print("Failed to set DNS suffix")
                    sys.exit(1)
                    
            else:
                # Show adapter info
                adapters = dns_manager.get_network_adapters()
                adapter = next(
                    (a for a in adapters if a["adapter_id"] == args.adapter),
                    None
                )
                
                if adapter:
                    print(format_adapter_info(adapter, args.format))
                else:
                    print(f"Adapter not found: {args.adapter}")
                    sys.exit(1)
                    
        else:
            parser.print_help()
            
    except Exception as e:
        logger.error(f"An error occurred: {e}")
        sys.exit(1)

if __name__ == "__main__":
    main()
